from msteams.adaptivecard.containers.layout import Layout


class FactSet(Layout):
    """
    FactSet element displays a series of facts (i.e. name/value pairs) in a tabular form
    """

    FACTS = 'facts'

    def __init__(self, spacing=None, separator=None):
        """
        :param spacing: amount of spacing
        :param separator: draw a separating line at the top of the element
        """
        Layout.__init__(self, 'FactSet')
        if spacing is not None:
            self.layout['spacing'] = spacing
        if separator is not None:
            self.layout['separator'] = separator

    def fact(self, fact):
        """Fact as a key/value pair
        :param fact:
        :return:
        """
        if self.FACTS not in self.layout.keys():
            self.layout[self.FACTS] = list()
        self.layout[self.FACTS].append(fact)
        return self



from msteams.adaptivecard.elements.element import Element


class Image(Element):
    """
    Displays an image
    """

    def __init__(self, url):
        """
        :param url: The URL to the image.
        :type url: str
        """
        Element.__init__(self, 'Image')
        self.element['url'] = url

    def style(self, style):
        """Controls how this Image is displayed
        :param style:
        :type style: str
        :return:
        """
        self.element['style'] = style
        return self

    def size(self, size):
        """approximate size of the image
        :param size: Controls the approximate size of the image.
        The physical dimensions will vary per host. Specify "auto" for true image dimension, or "stretch" to force it to fill the container.
        :type size: str
        :return:
        """
        self.element['size'] = size
        return self



from msteams.adaptivecard.containers.layout import Layout


class Fact(Layout):
    """
    Describes a Fact in a FactSet as a key/value pair.
    """

    def __init__(self, title, value):
        Layout.__init__(self, 'Fact')
        self.layout['title'] = title
        self.layout['value'] = value



from jsonpickle import encode


class AdaptiveCard:
    """
    Root element in an Adaptive Card
    """
    BODY = 'body'

    def __init__(self):
        self.card = dict()
        self.card['type'] = 'AdaptiveCard'
        self.card['$schema'] = 'http://adaptivecards.io/schemas/adaptive-card.json'
        self.card['version'] = '1.0'

    def container(self, container):
        """Containers group items together
        :param container:
        :return:
        """
        if self.BODY not in self.card.keys():
            self.card[self.BODY] = list()
        self.card[self.BODY].append(container)
        return self

    def build(self):
        return encode(self.card, unpicklable=False)



from msteams.adaptivecard.elements.element import Element


class TextBlock(Element):
    """
    Displays text, allowing control over font sizes, weight, and color
    """

    def __init__(self, text):
        """
        :param text:
        :type text: str
        """
        Element.__init__(self, 'TextBlock')
        self.element['text'] = text

    def spacing(self, spacing):
        """
        :param spacing: Controls the amount of spacing between this element and the preceding element.
        :type spacing: str
        :return:
        """
        self.element['spacing'] = spacing
        return self

    def size(self, size):
        """
        :param size: Controls size of text.
        :type size: str
        :return:
        """
        self.element['size'] = size
        return self

    def weight(self, weight):
        """weight of TextBlock elements
        :param weight: Controls the weight of TextBlock elements.
        :type weight: str
        :return:
        """
        self.element['weight'] = weight
        return self

    def wrap(self, wrap):
        """allow text to wrap
        :param wrap: If true, allow text to wrap. Otherwise, text is clipped.
        :type wrap: bool
        :return:
        """
        self.element['wrap'] = wrap
        return self

    def separator(self, separator):
        """draw a separating line
        :param separator: When true, draw a separating line at the top of the element.
        :type separator: bool
        :return:
        """
        self.element['separator'] = separator
        return self

    def is_subtle(self, is_subtle):
        """text slightly toned down to appear less prominent
        :param is_subtle: If true, displays text slightly toned down to appear less prominent.
        :type is_subtle: bool
        :return:
        """
        self.element['isSubtle'] = is_subtle
        return self

    def build(self):
        return self.element




class Section:
    """
    Sections to include in the card.
    """
    FACTS = 'facts'

    def __init__(self):
        self.section = dict()

    def activity_group(self, activity_image=None, activity_title=None, activity_subtitle=None, activity_text=None):
        """Activity group displayed in a two column layout
        :param activity_image: display the picture of that person
        :type activity_image: str
        :param activity_title: summarize what they did. Make it short and to the point
        :type activity_title: str
        :param activity_subtitle: subdued content
        :type activity_subtitle: str
        :param activity_text: details about the activity
        :type activity_text: str
        :return:
        """
        if activity_image is not None:
            self.section['activityImage'] = activity_image
        if activity_title is not None:
            self.section['activityTitle'] = activity_title
        if activity_subtitle is not None:
            self.section['activitySubtitle'] = activity_subtitle
        if activity_text is not None:
            self.section['activityText'] = activity_text
        return self

    def fact(self, fact):
        """name/value pair of fact
        :param fact: Facts are a very important component of a section. They often contain the information that really matters to the user.
        :return:
        """
        if self.FACTS not in self.section.keys():
            self.section[self.FACTS] = list()
        self.section[self.FACTS].append(fact)
        return self

    def build(self):
        return self.section



from jsonpickle import encode


class MessageCard:
    """
    Root element in an Message Card
    """
    SECTIONS = 'sections'

    def __init__(self):
        self.card = dict()
        self.card['@type'] = 'MessageCard'
        self.card['@context'] = 'https://schema.org/extensions'

    def title(self, title):
        """content of the card
        :param title: The title property is meant to be rendered in a prominent way, at the very top of the card.
        Use it to introduce the content of the card in such a way users will immediately know what to expect.
        :type title: str
        :return:
        """
        self.card['title'] = title
        return self

    def summary(self, summary):
        """determine what the card is all about
        :param summary: The summary property is typically displayed in the list view in Outlook,
        as a way to quickly determine what the card is all about
        :type summary: str
        :return:
        """
        self.card['summary'] = summary
        return self

    def theme_color(self, theme_color):
        """brand color for the card
        :param theme_color: Specifies a custom brand color for the card. The color will be displayed in a non-obtrusive manner.
        :type theme_color: str
        :return:
        """
        self.card['themeColor'] = theme_color
        return self

    def section(self, section):
        """sections to include in the card
        :param section:
        :type section: dict
        :return:
        """
        if self.SECTIONS not in self.card.keys():
            self.card[self.SECTIONS] = list()
        self.card[self.SECTIONS].append(section)
        return self

    def build(self):
        return encode(self.card, unpicklable=False)



import os.path
from setuptools import setup, find_packages

VERSION = "0.1.0"
HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, "README.md")) as fid:
    README = fid.read()

setup(
    name="ms-teams",
    version=VERSION,
    description="Helper library to construct microsoft teams connector cards",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/HarshadRanganathan/ms-teams",
    author="Harshad Ranganathan",
    author_email="rharshad93@gmail.com",
    license="MIT",
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="Microsoft Teams",
    packages=find_packages(exclude=["*.tests", "*.tests.*", "tests.*", "tests", "images"]),
    install_requires=[
        "jsonpickle"
    ],
    python_requires='>=3.7',
)



from msteams.adaptivecard.containers.layout import Layout


class Container(Layout):
    """
    Containers group items together
    """
    Items = 'items'

    def __init__(self):
        Layout.__init__(self, 'Container')

    def element(self, element):
        """card elements to render inside the Container.
        :param element: card element
        :return:
        """
        if self.Items not in self.layout.keys():
            self.layout[self.Items] = list()
        self.layout[self.Items].append(element)
        return self



from msteams.adaptivecard.containers.layout import Layout


class ColumnSet(Layout):
    """
    ColumnSet divides a region into Columns, allowing elements to sit side-by-side
    """
    COLUMNS = 'columns'

    def __init__(self, spacing=None, separator=None):
        Layout.__init__(self, 'ColumnSet')
        if spacing is not None:
            self.layout['spacing'] = spacing
        if separator is not None:
            self.layout['separator'] = separator

    def column(self, column):
        """container that is part of a ColumnSet.
        :param column:
        :return:
        """
        if self.COLUMNS not in self.layout.keys():
            self.layout[self.COLUMNS] = list()
        self.layout[self.COLUMNS].append(column)
        return self




class Fact:
    """
    Describes a Fact as a key/value pair.
    """

    def __init__(self, name, value):
        self.fact = dict()
        self.fact['name'] = name
        self.fact['value'] = value

    def build(self):
        return self.fact
